package com.yazoo.browser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import com.yazoo.browser.editors.WebBrowser;
import com.yazoo.browser.views.BrowserView;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "com.yazoo.browser";

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		
		// initialize the xulrunner bundle, then call new Browser(parent, SWT.MOZILLA);
		//initializeXulRunner();
	}
	
	private void initializeXulRunner() {
		Bundle bundle = Platform.getBundle("org.mozilla.xulrunner.win32.win32.x86"); 
		if (bundle != null) {
			URL resourceUrl = bundle.getResource("xulrunner"); 
			if (resourceUrl != null) {
				try {
					URL fileUrl = FileLocator.toFileURL(resourceUrl);
					File file = new File(fileUrl.toURI());
					System.setProperty("org.eclipse.swt.browser.XULRunnerPath", file.getAbsolutePath()); 
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
    public static Image getImage(String path) {
    	ImageDescriptor desc = getImageDescriptor(path);
        Image image = desc.createImage();
        return image;
    }
	
    public static IWorkbenchWindow getActiveWorkbenchWindow() {
        return getDefault().getWorkbench().getActiveWorkbenchWindow();
    }
	
    public static Shell getActiveWorkbenchShell() {
        IWorkbenchWindow window = getActiveWorkbenchWindow();
        if (window != null) {
            return window.getShell();
        }
        return null;
    }
    
	public static File getFileFromPlugin(String filePath) {
		File file = null;

		try {

			String pluginPath = FileLocator.resolve(getDefault().getBundle().getEntry("/")).getPath();
			StringBuffer sb = new StringBuffer();
			sb.append(pluginPath);
			sb.append("/");
			sb.append(filePath);

			IPath path = new Path(sb.toString());
			file = new File(path.toOSString());
			if (!file.exists()) {
				file = null;
			}

		} catch (IOException e) {
			e.printStackTrace();
		}

		return file;
	}
	
	public static String getScriptText(File file) {
		StringBuilder sb = new StringBuilder();
		
		try {

         	FileInputStream fis = new FileInputStream(file);
			InputStreamReader streamReader = new InputStreamReader(fis);
            BufferedReader bufferReader = new BufferedReader(streamReader);
             
            String line;
            while((line = bufferReader.readLine()) != null) {
                //System.out.println("line: " + line);
             	sb.append(line);
             	sb.append("\n");
            }

            bufferReader.close();
            streamReader.close();
            fis.close();
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return sb.toString();
	}
	
    public WebBrowser getWebBrowser() {
    	WebBrowser webBrowser = null;
        
        IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
        IEditorPart editorPart = page.getActiveEditor();

        if (editorPart instanceof WebBrowser) {
            webBrowser = (WebBrowser) editorPart;
        }
    	
    	return webBrowser;
    }
	
    public BrowserView getBrowserView() {
    	BrowserView browserView = null;

		IWorkbenchPage page = getActiveWorkbenchWindow().getActivePage(); 
		IViewReference viewReference = page.findViewReference(BrowserView.ID);
		if (viewReference != null) {
			try {
				IViewPart viewPart = page.showView(BrowserView.ID);
				page.activate(viewPart);
				browserView = (BrowserView) viewPart;
			} catch (PartInitException e) {
				e.printStackTrace();
			}
		}
    	
    	return browserView;
    }
}
