package com.yazoo.browser;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

import com.yazoo.browser.actions.OpenBrowserAction;
import com.yazoo.browser.actions.RunScriptAction;
import com.yazoo.browser.actions.SetSourceAction;
import com.yazoo.browser.actions.ShowSourceAction;

/**
 * An action bar advisor is responsible for creating, adding, and disposing of
 * the actions added to a workbench window. Each window will be populated with
 * new actions.
 */
public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	private IWorkbenchAction exitAction;
	
	private IWorkbenchAction aboutAction;
	
	private IAction showSourceAction;
	
	private IAction setSourceAction;
	
	private IAction runScriptAction;
	
	private IAction openBrowserAction;

	public ApplicationActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}

	protected void makeActions(final IWorkbenchWindow window) {

		exitAction = ActionFactory.QUIT.create(window);
		register(exitAction);
		
		aboutAction = ActionFactory.ABOUT.create(window);
		register(aboutAction);
		
		showSourceAction = new ShowSourceAction();
		
		setSourceAction = new SetSourceAction();
		
		runScriptAction = new RunScriptAction();
		
		openBrowserAction = new OpenBrowserAction();
	}

	protected void fillMenuBar(IMenuManager menuBar) {
		MenuManager fileMenu = new MenuManager("&File", IWorkbenchActionConstants.FILE_START);
		menuBar.add(fileMenu);
		fileMenu.add(exitAction);
		
		MenuManager helpMenu = new MenuManager("&Help", IWorkbenchActionConstants.M_HELP);
		menuBar.add(helpMenu);
        helpMenu.add(aboutAction);
	}
	
	protected void fillCoolBar(ICoolBarManager coolBar) {
	    IToolBarManager toolbar = new ToolBarManager(SWT.FLAT | SWT.RIGHT); 
        coolBar.add(new ToolBarContributionItem(toolbar, "main")); 
        toolbar.add(new Separator());
        toolbar.add(runScriptAction);
        toolbar.add(setSourceAction);
        toolbar.add(showSourceAction);
        toolbar.add(openBrowserAction);
    }

}
