package com.yazoo.browser.dialogs;

import java.io.File;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.yazoo.browser.Activator;
import com.yazoo.browser.utils.FileUtil;

/**
 * Dialog runs JavaScript against the current page of the SWT Browser widget.
 * 
 * @author Shawn Spiars
 *
 */
public class JavaScriptDialog extends TitleAreaDialog {
   
	private Text scriptText;
	
	private Text resultText;
	
	private String scriptString;
	
	private Browser browser;
	
	private Combo javaScriptCombo;
	
	public JavaScriptDialog(Shell shell, Browser browser) {
        super(shell);
        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.MAX);
        this.browser = browser;
    }

    protected Control createDialogArea(final Composite parent) {
        setTitle("Evaluate JavaScript");
        setMessage("Evaluate JavaScript commands against the current web page document.");
        
        GridLayout layout = new GridLayout(1, false);
		layout.marginHeight = 5;
		layout.marginWidth = 5;
		layout.horizontalSpacing = 8;
		layout.verticalSpacing = 8;
		parent.setLayout(layout);
         
        SashForm sash = new SashForm(parent, SWT.VERTICAL);
  		layout = new GridLayout(1, false);
		layout.marginHeight = 5;
		layout.marginWidth = 5;
		layout.horizontalSpacing = 8;
		layout.verticalSpacing = 8;
		sash.setLayout(layout);
		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.widthHint = 800;
		data.heightHint = 500;
		sash.setLayoutData(data);
        
        createScriptComposite(sash);
        
        createResultComposite(sash);
        
        sash.SASH_WIDTH=5;
	    sash.setWeights(new int[] {2, 1});
	    
	    loadScriptCombo();
		
		return sash;
    }
    
    private void createScriptComposite(Composite parent) {
		Group group = new Group(parent, SWT.NONE);
		group.setText("JavaScript Commands");
 		GridLayout layout = new GridLayout(2, false);
		layout.marginWidth = 5;
		layout.marginHeight = 4;
		layout.horizontalSpacing = 10;
		layout.verticalSpacing = 5;
		group.setLayout(layout);
		GridData data = new GridData(GridData.FILL_BOTH);
		group.setLayoutData(data);
		
		Composite buttonsLeft = new Composite(group, SWT.NONE);
		layout = new GridLayout(3, false);
		layout.horizontalSpacing = 10;
		buttonsLeft.setLayout(layout);
		buttonsLeft.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		
		Button browseButton = new Button(buttonsLeft, SWT.PUSH);
		browseButton.setToolTipText("Browse local file system for JavaScript files");
		browseButton.setImage(Activator.getImage("icons/folder.gif"));
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FileDialog dialog = new FileDialog(Activator.getActiveWorkbenchShell(), SWT.OPEN);
				dialog.setText("Select an image file");
				//dialog.setFilterPath(dirPath);
				dialog.setFilterExtensions(new String[] { "*.js;" });
				dialog.setFilterNames(new String[] { "(js)" });
				String fileName = dialog.open();
				File file = new File(fileName);
				try {
					String contents = FileUtil.getFileContents(file);
					scriptText.setText(contents);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		
		Button clearButton = new Button(buttonsLeft, SWT.PUSH);
		clearButton.setToolTipText("Clear");
		clearButton.setImage(Activator.getImage("icons/clear_co.gif"));
		clearButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				scriptText.setText("");
			}
		});
		
		Button runButton = new Button(buttonsLeft, SWT.PUSH);
		runButton.setToolTipText("Evaluate JavaScript commands");
		runButton.setImage(Activator.getImage("icons/run.gif"));
		runButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (scriptText.getText().trim().length() == 0) {
					setErrorMessage("JavaScript text is required!");
					scriptText.setFocus();
					return;
				}
				resultText.setText("");
				setErrorMessage(null);
				String script = scriptText.getText().trim();
				try {
					Object result = browser.evaluate(script);
					System.out.println("Script result: " + result);
					resultText.setText(result.toString());
				}
				catch(final Exception e) {
					//resultText.setText("JavaScript execution was unsuccessful!\n");
					//resultText.append(e.getMessage());
				}
			}
		});
		
		
		Composite buttonsRight = new Composite(group, SWT.NONE);
		layout = new GridLayout(2, false);
		layout.horizontalSpacing = 10;
		buttonsRight.setLayout(layout);
		buttonsRight.setLayoutData(new GridData(GridData.END, SWT.NONE, true, false));
		
		Label label = new Label(buttonsRight, SWT.NONE);
		label.setText("Sample Scripts:");
		
		javaScriptCombo = new Combo(buttonsRight, SWT.BORDER | SWT.READ_ONLY);
		javaScriptCombo.setLayoutData(new GridData(GridData.FILL, SWT.NONE, true, false));
		javaScriptCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String scriptName = javaScriptCombo.getText();
				File file = Activator.getFileFromPlugin("scripts/" + scriptName);
				String value = Activator.getScriptText(file);
				if (value != null) {
					scriptText.setText(value);
				}
			}
		});
		
		scriptText = new Text(group, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.horizontalSpan = 2;
		scriptText.setLayoutData(data);
    }

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL, false);
	}
    
    private void createResultComposite(Composite parent) {
		Group group = new Group(parent, SWT.NONE);
		group.setText("Results");
 		GridLayout layout = new GridLayout(1, false);
		//layout.marginTop = 2;
		layout.marginWidth = 5;
		layout.marginHeight = 4;
		layout.horizontalSpacing = 10;
		layout.verticalSpacing = 5;
		group.setLayout(layout);
		GridData data = new GridData(GridData.FILL_BOTH);
		group.setLayoutData(data);
		
		resultText = new Text(group, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		data = new GridData(SWT.FILL, SWT.FILL, true, true);
		resultText.setLayoutData(data);
    }
    
    private void loadScriptCombo() {
    	File dir = FileUtil.getFolderInRoot("scripts");
    	File[] dirFiles = dir.listFiles();
    	for(File f : dirFiles) {
    		javaScriptCombo.add(f.getName());
    	}
    }
    
    public String getScriptString() {
    	return scriptString;
    }
    
	public void setFocus() {
		scriptText.setFocus();
	}


    
}