package com.yazoo.browser.dialogs;

import java.io.File;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.yazoo.browser.Activator;
import com.yazoo.browser.utils.CustomFunction;
import com.yazoo.browser.utils.FileUtil;

/**
 * Dialog sets the contents of the SWT Browser widget.
 * 
 * @author Shawn Spiars
 *
 */
public class SetSourceDialog extends TitleAreaDialog {
   
	private Text sourceText;
	
	private Combo htmlCombo;
	
	private Browser browser;
	
	public SetSourceDialog(Shell shell, Browser browser) {
        super(shell);
        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.MAX);
        this.browser = browser;
    }

    protected Control createDialogArea(final Composite parent) {
        setTitle("Set Browser Contents");
        setMessage("Enter HTML content to display in the web browser and press OK.");
         
        final Composite composite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout(2, false);
		layout.marginTop = 2;
		layout.marginBottom = 2;
		composite.setLayout(layout);
		
		GridData data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);
		
		Label label = new Label(composite, SWT.NONE);
		label.setText("Sample HTML:");
		
		htmlCombo = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);
		htmlCombo.setLayoutData(new GridData(GridData.FILL, SWT.NONE, true, false));
		htmlCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String scriptName = htmlCombo.getText();
				if (scriptName.equals("snippet307.html")) {
					BrowserFunction function = new CustomFunction (browser, "theJavaFunction");
				}
				File file = Activator.getFileFromPlugin("html/" + scriptName);
				String value = Activator.getScriptText(file);
				if (value != null) {
					sourceText.setText(value);
				}
			}
		});
 	
		sourceText = new Text(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.horizontalSpan = 2;
		data.widthHint = 700;
		data.heightHint = 500;
		sourceText.setLayoutData(data);
		
		loadHtmlCombo();
		
		return composite;
    }
    
    private void loadHtmlCombo() {
    	File dir = FileUtil.getFolderInRoot("html");
    	File[] dirFiles = dir.listFiles();
    	for(File f : dirFiles) {
    		htmlCombo.add(f.getName());
    	}
    }
    
	public void buttonPressed(final int id) {
		if (id == IDialogConstants.OK_ID) {
			if (sourceText.getText().trim().length() == 0) {
				setErrorMessage("Please provide HTML contents for page.");
				sourceText.setFocus();
				return;	
			}

			browser.setText(sourceText.getText().trim());
			
		}
		super.buttonPressed(id);
	}
    
}